#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h> 
#include <stdio.h> 
#include <stdint.h>
#include "Nokia5110/image.h"

#include "Nokia5110/Nokia5110.h"
#include "dht.h"
#include "temt6000.h"

static int cursor = 0;
static int old_cursor = 0;
int e; //ENTER

struct flower {
	char *name;
	int min_temp;
	int max_temp;
	int light;
	int hummidity;
	int bad_temp;
	int bad_light;
	int bad_humm; /* procente */
};

static struct flower flowers[4];


// Defines 
#define LCD_WIDTH  84
#define LCD_HEIGHT 48

#define BLOCK_SIZE 4

#define LIGHT_SENSOR 0
#define HUMIDITY_SOIL_SENSOR 1

uint8_t channelZero = LIGHT_SENSOR;

/* Valoare citita canal 0*/
volatile uint16_t ADC_value;


ISR(PCINT1_vect)
{
    if ((PINB & (1 << PB3)) == 0)
    {
        cursor--;	
 		if(cursor == -1)
 			cursor = 3;

 		LCD_write_string(0, cursor, "->");
 		_delay_ms(50);
 		if(old_cursor != cursor)
 			LCD_write_string(0, old_cursor, "  ");
	

 		old_cursor = cursor;

    }

     if ((PINB & (1 << PB4)) == 0)
    {
        cursor++;	
 		if(cursor == 4)
 			cursor = 0;

 		LCD_write_string(0, cursor, "->");
 		_delay_ms(50);

 		if(old_cursor != cursor)
 			LCD_write_string(0, old_cursor, "  ");
	

 		old_cursor = cursor;

    }
    _delay_ms(100);
}

void ADC_init(void)
{
    ADMUX = 0;
    ADMUX |= (1 << REFS0);
    ADCSRA = (1 << ADEN) | (7 << ADPS0);

}

/*
    Returns the result of an ADC conversion. 
    Starts a conversion and waits for the result when not using interrupts.
*/
int ADC_get(uint8_t channel)
{
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

/*
    Converts a value given by the ADC to volts.
*/
double ADC_voltage(int raw)
{
    (void)raw;

    return raw * 5 / 1024.;
}

long map( long x,  long in_min,  long in_max,  long out_min,  long out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

void led_init()
{
	DDRC |= (1 << PC6);
	DDRC |= (1 << PC5);
	PORTC |= (1 << PC5);
	DDRC |= (1 << PC4);
}
void buttons_init()
{
    DDRB &= ~(1 << PB2);
    PORTB |= (1 << PB2);

    DDRB &= ~(1 << PB3);
    PORTB |= (1 << PB3);

    DDRB &= ~(1 << PB4);
    PORTB |= (1 << PB4);
    // Activez PIN CHANGE INTERRUPT ENABLE pentru PCINT15:8 (portul B)
    // Orice PIN setat pt intrerupere va semnala programul
    PCICR |= (1 << PCIE1);

    //Activez intreruperea pe PB3 si PB4
    PCMSK1 |= (1 << PCINT11);
    PCMSK1 |= (1 << PCINT12);
   
}
void init_params()
{
	int i;

	flowers[0].name = "Camellia";
	flowers[0].min_temp = 7;
	flowers[0].max_temp = 18;
	flowers[0].light = 500;
	flowers[0].hummidity = 50;

	flowers[1].name = "Freesia";
	flowers[1].min_temp = 12;
	flowers[1].max_temp = 18;
	flowers[1].light = 500;
	flowers[1].hummidity = 55;

	flowers[2].name = "Petunia";
	flowers[2].min_temp = 20;
	flowers[2].max_temp = 30;
	flowers[2].light = 500;
	flowers[2].hummidity = 40;

	flowers[3].name = "Primula";
	flowers[3].min_temp = 10;
	flowers[3].max_temp = 15;
	flowers[3].light = 250;
	flowers[3].hummidity = 50;

	for(i = 0; i <= 3; i++){
		flowers[i].bad_temp = 0;
		flowers[i].bad_light = 0;
		flowers[i].bad_humm = 0;
	}

}


void comparison(int temp, int lux, int h)
{
	int i;

	LCD_write_string(0, 3, "             ");
	LCD_write_string(0, 4, "             ");
  	LCD_write_string(0, 5, "             ");
  	PORTC &= ~(1 << PC6);
  	PORTC &= ~(1 << PC4);

  	for(i = 0; i <= 3; i++){
		flowers[i].bad_temp = 0;
		flowers[i].bad_light = 0;
		flowers[i].bad_humm = 0;
	}

	if(temp < flowers[cursor].min_temp){
		LCD_write_string(0, 3, "Cam frig!");
		flowers[cursor].bad_temp = 1;

	}
	else if (temp > flowers[cursor].max_temp){
		LCD_write_string(0, 3, "Cam cald!");
		flowers[cursor].bad_temp = 1;

	}
	if ((h - flowers[cursor].hummidity) > 25){
		LCD_write_string(0, 4, "Multa apa!");
		flowers[cursor].bad_humm = 1;
	}
	else if ((flowers[cursor].hummidity - h) > 25){
		LCD_write_string(0, 4, "Putina apa!");
		flowers[cursor].bad_humm = 1;
	}

	if(lux - flowers[cursor].light > 400){
		LCD_write_string(0, 5, "Multa lumina!");
		flowers[cursor].bad_light = 1;

	}	
	else if(flowers[cursor].light - lux > 400){
		LCD_write_string(0, 5, "Putina lumina!");
		flowers[cursor].bad_light = 1;
	}
	if(flowers[cursor].bad_temp == 0 && flowers[cursor].bad_humm == 0
		&& flowers[cursor].bad_light == 0){
		PORTC |= (1 << PC5);

	}	
	if(flowers[cursor].bad_temp == 1 || flowers[cursor].bad_humm == 1
		|| flowers[cursor].bad_light == 1){
		PORTC |= (1 << PC4);
		PORTC &= ~(1 << PC5);

	}
	if(flowers[cursor].bad_temp == 1 && flowers[cursor].bad_humm == 1
		&& flowers[cursor].bad_light == 1){
		PORTC |= (1 << PC6);
		PORTC &= ~(1 << PC5);
		PORTC &= ~(1 << PC4);

	}
}

int enter()
{
	if ((PINB & (1 << PB2)) == 0) 
        return 1;

    return 0;
}


int main(void) 
{
	int8_t temperature = 0;
	int8_t hummidity = 0;
	char temp[8];
	uint16_t adc = 0;
	char printbuff[100];
	char value_voltage[32];
	char value_lux[32];
	char value_hsoil[32];
	long h_soil_percent;
	int ef = 0;

	
	LCD_init();    
	LCD_clear();

	sei();
	buttons_init();
	init_params();
	led_init();

	ADC_init();

	LCD_draw_bmp_pixel(0,0,image1,LCD_WIDTH,LCD_HEIGHT);
	LCD_write_string(0,2,"E");
	LCD_write_string(0,3,"V");
	LCD_write_string(0,4,"E");
	LCD_write_string(0,5,"R");

	LCD_write_string(76,1,"B");
	LCD_write_string(76,2,"L");
	LCD_write_string(76,3,"O");
	LCD_write_string(76,4,"O");
	LCD_write_string(76,5,"M");

	while(1) {
		e = enter();
		if(e == 1){
			break;
		}
	}
	e = 0;
	LCD_clear();
	LCD_write_string(13, 0, "Camellia");
	LCD_write_string(13, 1, "Freesia");
	LCD_write_string(13, 2, "Petunia");
	LCD_write_string(13, 3, "Primula");


	while(1) {
		e = enter();
		if(e == 1){
			break;
		}
	}
	e = 0;

	_delay_ms(5000);
	LCD_clear();

	while(1) {

		double voltage_lum = ADC_voltage(ADC_get(LIGHT_SENSOR));
		int h_soil = ADC_get(HUMIDITY_SOIL_SENSOR);
		h_soil = 1023 - h_soil;
		/* Mapeaza valorile umiditatii in procente*/
		h_soil_percent = map(h_soil, 0, 1023, 0, 100);

		double lux = temt6000_getlux(voltage_lum);

		_delay_ms(50);

		dht_gettemperaturehumidity(&temperature,&hummidity);
		_delay_ms(50);


		dtostrf(voltage_lum,3,5,value_voltage);
		dtostrf(lux,3,0,value_lux);
		itoa((int)h_soil_percent,value_hsoil,10);
		itoa(temperature,temp,10);
		
		comparison((int)temperature,(int)lux,h_soil_percent);

		LCD_write_string(0, 0, "T:");
		LCD_write_string(20, 0, temp);
		LCD_write_string(35, 0, "Celsius");


		LCD_write_string(0, 1, "L:");
		LCD_write_string(20, 1,value_lux);
		LCD_write_string(40, 1, "lux");


		LCD_write_string(0, 2, "H:");
		LCD_write_string(20, 2, value_hsoil);
		LCD_write_string(40, 2, "%");





    }
	
	LCD_clear();

	return 0;
}



